# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""WSGI, Celery and CLI applications for Invenio flavours.

Invenio-App provides the default Flask, WSGI, Celery and CLI applications which
is needed in order to run Invenio. All which is needed to run your Inveno
instance is to provide a base configuration should via the
``invenio_config.module`` entry point.

Setting up your instance
------------------------
Your base configuration e.g. defines the default language, name of your site as
well as the data model you are using. Usually this base configuration is put in
Python module inside a package. For instance like this (assuming your instance
is called ``mysite``)

.. code-block:: python

    # mysite/config.py

    BABEL_DEFAULT_LANGUAGE = 'en'
    BABEL_DEFAULT_TIMEZONE = 'Europe/Zurich'
    # ...

In order to tell the Invenio-App applications to load your base module, you
should set the config module import path in the ``invenio_config.module`` entry
point group like the example below:

.. code-block:: python

    # setup.py
    setup(
        # ...
        entry_points={
            'invenio_config.module': [
                'mysite = mysite.config',
            ],
        }
    )

The entry point ``'mysite = mysite.config'`` defines an entry point named
``mysite`` pointing to the Python module ``mysite.config`` (i.e. the config
module above).

After you edited setup.py, you should always remember to install your Python
package again, as otherwise the newly added entry points won't be picked up.

Applications
------------

CLI
~~~
The command-line interface application is named ``invenio`` and you use it to
e.g. run a development server or initialize the database:

.. code-block:: console

    $ invenio --help
    Usage: invenio [OPTIONS] COMMAND [ARGS]...

      Command Line Interface for Invenio.

    ...

Celery
~~~~~~
In order to start a Celery worker for Invenio, you need to point Celery to the
Invenio Celery application (``invenio_app.celery``) in the following manner:

.. code-block:: console

    $ celery worker --app invenio_app.celery --loglevel INFO

WSGI
~~~~
Python web applications are usually run by a WSGI server such as Gunicorn,
UWSGI or mod_wsgi for Apache. Similar to Celery, you need to point the WSGI
servers to the Invenio WSGI application.

Here is e.g. an example with `Gunicorn <http://gunicorn.org>`_:

.. code-block:: console

    $ pip install gunicorn
    $ gunicorn invenio_app.wsgi

Invenio-App provides three different WSGI applications depending on your needs:

- ``invenio_app.wsgi``: Combined UI + REST API application with the REST API
  mounted under ``/api``.
- ``invenio_app.wsgi_ui``: UI-only application.
- ``invenio_app.wsgi_rest``: REST API-only application.

The individual UI and REST API applications are useful if you want to run
the UI and API on different servers or domains (e.g. ``www.example.org`` and
``api.example.org``), whereas the combined application is useful if you want to
run everything on the same server (e.g. ``www.example.org`` and
``www.example.org/api/``).

Deployment
----------
Deploying the applications in a production environment among other things
involve:

- daemonizing the applications
- setting instance specific configuration (e.g. database hostname)

Daemonizing the applications
~~~~~~~~~~~~~~~~~~~~~~~~~~~~
In a production system you would usually need to daemonize the WSGI and Celery
application. A full guide on doing this is outside the scope of this
documentation, but usually it involves setting up e.g.
`Supervisord <http://supervisord.org>`_ or another process management tool to
manage the automatic starting/stopping ot the application.

For a full guide on how to deploy Invenio, please see the general
`Invenio documentation <http://invenio.readthedocs.io>`_.

Instance configuration
~~~~~~~~~~~~~~~~~~~~~~
The instance configuration defines e.g. hostnames of your database server and
other values which change depending on where you are running the applications
(e.g. local development machine vs. production system). In comparison the base
configuration is the same no matter where you run the application (e.g. site
name).

The instance configuration can be provided either in
``<instance-path>/invenio.cfg`` or via environment variables prefixed with
``INVENIO_``, e.g.:

.. code-block:: console

   $ export INVENIO_SQLALCHEMY_DATABASE_URI = ...

The instance path is defined by the environment variable
``INVENIO_INSTANCE_PATH`` or if not set defaults to
``<sys.prefix>/var/instance/`` where <sys.prefix> is your Python root prefix
(e.g. ``/usr/``)

Templates and static files
~~~~~~~~~~~~~~~~~~~~~~~~~~
You can add templates and static files in the following folders respectively:

- ``<instance-path>/templates/``
- ``<instance-path>/static/``

This should only be done for small number of templates/files, as it is usually
better to provide them via an installabled Python package. Do take care not
to overwrite any existing Invenio files.
"""

from __future__ import absolute_import, print_function

from .ext import InvenioApp
from .version import __version__

__all__ = ('__version__', 'InvenioApp', )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

from __future__ import absolute_import, print_function

import pytest
from flask import Flask

from invenio_app.config import APP_DEFAULT_SECURE_HEADERS


@pytest.fixture()
def base_app():
    """Flask application fixture."""
    app_ = Flask('testapp')
    app_.config.update(
        SECRET_KEY='SECRET_KEY',
        TESTING=True,
    )
    app_.config['APP_DEFAULT_SECURE_HEADERS'] = APP_DEFAULT_SECURE_HEADERS
    app_.config['APP_DEFAULT_SECURE_HEADERS']['force_https'] = False
    return app_


@pytest.yield_fixture()
def app(base_app):
    """Flask application fixture."""
    with base_app.app_context():
        yield base_app


@pytest.fixture()
def wsgi_apps():
    """Wsgi app fixture."""
    from invenio_base.wsgi import create_wsgi_factory, wsgi_proxyfix
    from invenio_base.app import create_app_factory

    def _config(app, **kwargs):
        app.config.update(
            SECRET_KEY='SECRET_KEY',
            TESTING=True,
        )
        app.config['APP_DEFAULT_SECURE_HEADERS'] = APP_DEFAULT_SECURE_HEADERS
        app.config['APP_DEFAULT_SECURE_HEADERS']['force_https'] = False
    # API
    create_api = create_app_factory(
        'invenio',
        config_loader=_config,
        wsgi_factory=wsgi_proxyfix(),
    )
    # UI
    create_ui = create_app_factory(
        'invenio',
        config_loader=_config,
        wsgi_factory=wsgi_proxyfix(),
    )
    # Combined
    create_app = create_app_factory(
        'invenio',
        config_loader=_config,
        wsgi_factory=wsgi_proxyfix(create_wsgi_factory({'/api': create_api})),
    )
    return create_app, create_ui, create_api



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Module tests."""

from __future__ import absolute_import, print_function

from flask import request

from invenio_app.factory import create_ui


def test_version():
    """Test version import."""
    from invenio_app import __version__
    assert __version__


def test_config_loader():
    """Test config loader."""
    app = create_ui()
    assert 'cache_size' in app.jinja_options


def test_trusted_hosts():
    """Test trusted hosts configuration."""
    app = create_ui(
        APP_ALLOWED_HOSTS=['example.org', 'www.example.org'],
        APP_ENABLE_SECURE_HEADERS=False,
    )

    @app.route('/')
    def index():
        return request.host

    with app.test_client() as client:
        res = client.get('/', headers={'Host': 'attacker.org'})
        assert res.status_code == 400

        res = client.get('/', headers={'Host': 'example.org'})
        assert res.status_code == 200

        res = client.get('/', headers={'Host': 'www.example.org'})
        assert res.status_code == 200

    # Make sure X-Forwarded-Host can be used as well.
    with app.test_client() as client:
        res = client.get('/', headers={
            'Host': 'example.org',
            'X-Forwarded-Host': 'attacker.org'
        })
        assert res.status_code == 400



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio App configuration.

`Invenio-App` is partially overwriting default configuration of `Limiter` and
`Talisman` applications.
You can find below more details about which configuration are set.

For more information, please also see
`Flask-Limiter <https://flask-limiter.readthedocs.io/en/stable/>`_ and
`Flask-Talisman <https://github.com/GoogleCloudPlatform/flask-talisman/>`__
websites.
"""

RATELIMIT_DEFAULT = '5000/hour'
"""Default rate limit.

.. note:: Overwrite
   Flask-Limiter <https://flask-limiter.readthedocs.io/en/stable/>`_
   configuration.
"""

RATELIMIT_HEADERS_ENABLED = True
"""Enable rate limit headers. (Default: ``True``)

.. note:: Overwrite
   Flask-Limiter <https://flask-limiter.readthedocs.io/en/stable/>`_
   configuration.
"""

APP_ENABLE_SECURE_HEADERS = True
"""Enable Secure Headers. (Default: ``True``)

In case you want to disable completely `Talisman`, you can set to `False`.

Remember that, for development purpose, setting ```DEBUG = True``` is already
enough to disable any side effects such as force ``https``.

.. note::
    `W3C
    <https://www.w3.org/TR/CSP2/>`_
"""

APP_DEFAULT_SECURE_HEADERS = {
    'force_https': True,
    'force_https_permanent': False,
    'force_file_save': False,
    'frame_options': 'sameorigin',
    'frame_options_allow_from': None,
    'strict_transport_security': True,
    'strict_transport_security_preload': False,
    'strict_transport_security_max_age': 31556926,  # One year in seconds
    'strict_transport_security_include_subdomains': True,
    'content_security_policy': {
        'default-src': ["'self'"],
        'object-src': ["'none'"]
    },
    'content_security_policy_report_uri': None,
    'content_security_policy_report_only': False,
    'session_cookie_secure': True,
    'session_cookie_http_only': True
}
"""Talisman default Secure Headers configuration.

As default, invenio assumes that HTTPS is enabled.
If you are not using SSL, then remember to disable the `force_https` and
`session_cookie_secure` configuration options related to HTTPS.

Please note that, as default `Talisman` behaviour, if Flask `DEBUG` mode is on,
then also many security barriers are automatically switched off
(e.g. `force_https` and `session_cookie_secure`).

.. note:: Overwrite
    `Flask-Talisman
    <https://github.com/GoogleCloudPlatform/flask-talisman>`_ configuration.

.. code-block:: python

    from flask import Flask
    from flask_talisman import Talisman

    app = Flask(__name__)
    app.config.update(
        SECRET_KEY='SECRET_KEY'
    )
    talisman = Talisman(app)

    @app.route('/defenders')
    @talisman(frame_options_allow_from='*')
    def defenders():
        \"\"\"Override policies for the specific view.\"\"\"
        return 'Jessica Jones'
"""

APP_ALLOWED_HOSTS = None
"""A list of host/domain names that can be served.

This is a security measure to prevent HTTP Host header attacks, which are
possible even under many seemingly-safe web server configurations.

By default all hosts are allowed. Values in this list can be fully qualified
names (e.g. 'www.example.com'). The validation only applies to
``request.host``.

In addition to this configuration variable, you should make sure that your
web server does not route requests to the application with an invalid Host
header.
"""

APP_HEALTH_BLUEPRINT_ENABLED = True
"""Enable the ping (healthcheck) blueprint. (Default: ``False``)
"""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Helpers."""

from __future__ import absolute_import, print_function

from flask import current_app


class TrustedHostsMixin(object):
    """Mixin for reading trusted hosts from application config."""

    @property
    def trusted_hosts(self):
        """Get list of trusted hosts."""
        if current_app:
            return current_app.config.get('APP_ALLOWED_HOSTS', None)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2017-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Flask application factories for Invenio flavours."""

from __future__ import absolute_import, print_function

import os
import sys

import pkg_resources
from invenio_base.app import create_app_factory
from invenio_base.wsgi import create_wsgi_factory, wsgi_proxyfix
from invenio_cache import BytecodeCache
from invenio_config import create_config_loader
from jinja2 import ChoiceLoader, FileSystemLoader

from .helpers import TrustedHostsMixin

env_prefix = 'INVENIO'

invenio_config_loader = create_config_loader(
    config=None, env_prefix=env_prefix
)

instance_path = os.getenv(env_prefix + '_INSTANCE_PATH') or \
    os.path.join(sys.prefix, 'var', 'instance')
"""Instance path for Invenio.

Defaults to ``<env_prefix>_INSTANCE_PATH`` or if environment variable is not
set ``<sys.prefix>/var/instance``.
"""

static_folder = os.getenv(env_prefix + '_STATIC_FOLDER') or \
    os.path.join(instance_path, 'static')
"""Static folder path.

Defaults to ``<env_prefix>_STATIC_FOLDER`` or if environment variable is not
set ``<sys.prefix>/var/instance/static``.
"""


def config_loader(app, **kwargs_config):
    """Configuration loader.

    Adds support for loading templates from the Flask application's instance
    folder (``<instance_folder>/templates``).
    """
    # This is the only place customize the Flask application right after
    # it has been created, but before all extensions etc are loaded.
    local_templates_path = os.path.join(app.instance_path, 'templates')
    if os.path.exists(local_templates_path):
        # Let's customize the template loader to look into packages
        # and application templates folders.
        app.jinja_loader = ChoiceLoader([
            FileSystemLoader(local_templates_path),
            app.jinja_loader,
        ])

    app.jinja_options = dict(
        app.jinja_options,
        cache_size=1000,
        bytecode_cache=BytecodeCache(app)
    )

    invenio_config_loader(app, **kwargs_config)


def app_class():
    """Create Flask application class.

    Invenio-Files-REST needs to patch the Werkzeug form parsing in order to
    support streaming large file uploads. This is done by subclassing the Flask
    application class.
    """
    try:
        pkg_resources.get_distribution('invenio-files-rest')
        from invenio_files_rest.app import Flask as FlaskBase
    except pkg_resources.DistributionNotFound:
        from flask import Flask as FlaskBase

    # Add Host header validation via APP_ALLOWED_HOSTS configuration variable.
    class Request(TrustedHostsMixin, FlaskBase.request_class):
        pass

    class Flask(FlaskBase):
        request_class = Request

    return Flask


create_api = create_app_factory(
    'invenio',
    config_loader=config_loader,
    blueprint_entry_points=['invenio_base.api_blueprints'],
    extension_entry_points=['invenio_base.api_apps'],
    converter_entry_points=['invenio_base.api_converters'],
    wsgi_factory=wsgi_proxyfix(),
    instance_path=instance_path,
    app_class=app_class(),
)
"""Flask application factory for Invenio REST API."""

create_ui = create_app_factory(
    'invenio',
    config_loader=config_loader,
    blueprint_entry_points=['invenio_base.blueprints'],
    extension_entry_points=['invenio_base.apps'],
    converter_entry_points=['invenio_base.converters'],
    wsgi_factory=wsgi_proxyfix(),
    instance_path=instance_path,
    static_folder=static_folder,
    app_class=app_class(),
)
"""Flask application factory for Invenio UI."""

create_app = create_app_factory(
    'invenio',
    config_loader=config_loader,
    blueprint_entry_points=['invenio_base.blueprints'],
    extension_entry_points=['invenio_base.apps'],
    converter_entry_points=['invenio_base.converters'],
    wsgi_factory=wsgi_proxyfix(create_wsgi_factory({'/api': create_api})),
    instance_path=instance_path,
    static_folder=static_folder,
    app_class=app_class(),
)
"""Flask application factory for combined UI + REST API.

REST API is mounted under ``/api``.
"""
